require "import"
import "android.app.*"
import "android.content.*"
import "android.os.Vibrator"
import "android.speech.RecognizerIntent"
import "android.speech.tts.TextToSpeech"
import "android.net.Uri"
import "android.view.accessibility.AccessibilityNodeInfo"

local Locale = luajava.bindClass("java.util.Locale")
local tts
local vibrator

function initTTS()
  tts = TextToSpeech(activity, TextToSpeech.OnInitListener{
    onInit = function(status)
      if status == TextToSpeech.SUCCESS then
        tts.setLanguage(Locale("ro"))
      end
    end
  })
end

function vibrate()
  vibrator = activity.getSystemService(Context.VIBRATOR_SERVICE)
  if vibrator then
    vibrator.vibrate(300) -- Vibrație de 300ms înainte de dictare
  end
end

function speakText(text)
  if tts then tts.speak(text, TextToSpeech.QUEUE_FLUSH, nil, "ttsID") end
end

function startVoiceRecognition()
  vibrate() -- Vibrație înainte de dictare
  local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
  intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
  intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "ro-RO")
  intent.putExtra(RecognizerIntent.EXTRA_PROMPT, "Spune ce vrei să cauți pe Google...")
  activity.startActivityForResult(intent, 1)
end

function openChromeSearch(query)
  local url = "https://www.google.com/search?q=" .. query:gsub(" ", "+")
  local intent = Intent(Intent.ACTION_VIEW, Uri.parse(url))
  intent.setPackage("com.android.chrome") -- Se asigură că se deschide în Chrome
  activity.startActivity(intent)
  speakText("> Caut pe Google: " .. query)

  task(4000, function()
    clickFirstResult()
  end)
end

function clickFirstResult()
  local root = service.getRootInActiveWindow()
  if not root then return task(1000, clickFirstResult) end

  for i = 0, root.getChildCount() - 1 do
    local node = root.getChild(i)
    if node and node.getClassName() == "android.widget.TextView" and isValidResult(node) then
      node.performAction(AccessibilityNodeInfo.ACTION_CLICK)
      speakText("> Am deschis primul rezultat.")
      task(5000, readPageContent)
      return
    end
  end

  task(1000, clickFirstResult)
end

function isValidResult(node)
  local text = node.getText()
  return text and text:match("https?://") -- Verifică dacă este un link valid
end

function readPageContent()
  local root = service.getRootInActiveWindow()
  if not root then return end

  local textContent = ""
  for i = 0, root.getChildCount() - 1 do
    local node = root.getChild(i)
    if node and node.getClassName() == "android.widget.TextView" then
      textContent = textContent .. " " .. node.getText()
    end
  end

  if textContent ~= "" then
    speakText("> Pagina spune: " .. textContent:sub(1, 500)) -- Citește primele 500 de caractere
  else
    speakText("> Nu am găsit text de citit.")
  end
end

function onActivityResult(requestCode, resultCode, data)
  if requestCode == 1 and resultCode == -1 and data then
    local matches = data.getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS)
    if matches and matches.size() > 0 then
      openChromeSearch(matches.get(0))
    end
  end
end

function onCreate()
  initTTS()
  startVoiceRecognition()
end